package frc.robot.subsystems;

import com.revrobotics.AbsoluteEncoder;
import com.revrobotics.spark.SparkMax;
import com.revrobotics.spark.SparkLowLevel.MotorType;

import edu.wpi.first.math.controller.ElevatorFeedforward;
import edu.wpi.first.math.controller.PIDController;
import edu.wpi.first.wpilibj2.command.Command;
import edu.wpi.first.wpilibj2.command.SubsystemBase;
import frc.robot.constants.ElevatorConstants;

public class Elevator extends SubsystemBase {
    private SparkMax motor1;

    private AbsoluteEncoder encoder;

    private PIDController pidController;

    private ElevatorFeedforward feedForward;
    
    public Elevator() {
        motor1 = new SparkMax(
            ElevatorConstants.kMotor1ID, 
            MotorType.kBrushless
        );

        encoder = motor1.getAbsoluteEncoder();

        pidController = new PIDController(
            ElevatorConstants.kPIDControllerP, 
            ElevatorConstants.kPIDControllerI, 
            ElevatorConstants.kPIDControllerD
        );

        feedForward = new ElevatorFeedforward(
            ElevatorConstants.kFeedForwardS, 
            ElevatorConstants.kFeedForwardG, 
            ElevatorConstants.kFeedForwardV
        );
    }

    public Command runElevator(double speed) {
        return run(() -> {
            motor1.set(speed);
        });
    }
    
    public Command goToSetpoint(double setpoint) {
        return run(() -> {
            motor1.set(feedForward.calculate(
                encoder.getVelocity(), 
                pidController.calculate(
                    encoder.getPosition(), 
                    setpoint
                )
            ));
        });
    }
}
